@@ -1,3 +1,3 @@
 from pandasgui.gui import show
 
-__all__ = ['show']
+__all__ = ["show"]
