@@ -1,50 +1,31 @@
-"""
-Defines the DataFrameViewer class to display DataFrames as a table. The DataFrameViewer is made up of three separate
-QTableWidgets... DataTableView for the DataFrame's contents, and two HeaderView widgets for the column and index headers
-"""
-
-from PyQt5 import QtGui, QtCore, QtWidgets
-from PyQt5.QtCore import QAbstractItemModel, QModelIndex, QSize, QRect, Qt, QPoint, QItemSelectionModel
-from PyQt5.QtGui import QPainter, QFont, QFontMetrics, QPalette, QBrush, QColor, QTransform
-from PyQt5.QtWidgets import QSizePolicy
-import pandas as pd
-import numpy as np
 import sys
 import threading
-from pandasgui.store import store
+import os
 
-class DataFrameStore:
-    def __init__(self, df):
-        self.df = df
-        self.models = []
+import numpy as np
+import pandas as pd
+from PyQt5 import QtCore, QtGui, QtWidgets
+from PyQt5.QtCore import Qt
 
-    def update(self, df):
-        self.df = df
+from pandasgui.utility import get_logger
+from pandasgui.store import Store, PandasGuiDataFrame
+import pandasgui
 
+logger = get_logger(__name__)
 
 
 class DataFrameViewer(QtWidgets.QWidget):
-    """
-    Displays a DataFrame as a table.
-
-    Args:
-        df (DataFrame): The DataFrame to display
-    """
-
-    def __init__(self, df):
-
+    def __init__(self, pgdf: PandasGuiDataFrame):
         super().__init__()
+
+        pgdf = PandasGuiDataFrame.cast(pgdf)
+        pgdf.dataframe_viewer = self
+
+        self.pgdf = pgdf
+
         # Indicates whether the widget has been shown yet. Set to True in
         self._loaded = False
 
-        if not type(df) == pd.DataFrame:
-            orig_type = type(df)
-            df = df.to_frame()
-            print(f'DataFrame was automatically converted from {orig_type} to DataFrame for viewing')
-
-        # Put the df in a wrapper
-        self.df = df
-
         # Set up DataFrame TableView and Model
         self.dataView = DataTableView(parent=self)
 
@@ -52,17 +33,28 @@ class DataFrameViewer(QtWidgets.QWidget):
         self.columnHeader = HeaderView(parent=self, orientation=Qt.Horizontal)
         self.indexHeader = HeaderView(parent=self, orientation=Qt.Vertical)
 
+        self.columnHeaderNames = HeaderNamesView(parent=self, orientation=Qt.Horizontal)
+        self.indexHeaderNames = HeaderNamesView(parent=self, orientation=Qt.Vertical)
+
         # Set up layout
         self.gridLayout = QtWidgets.QGridLayout()
         self.setLayout(self.gridLayout)
 
         # Link scrollbars
         # Scrolling in data table also scrolls the headers
-        self.dataView.horizontalScrollBar().valueChanged.connect(self.columnHeader.horizontalScrollBar().setValue)
-        self.dataView.verticalScrollBar().valueChanged.connect(self.indexHeader.verticalScrollBar().setValue)
+        self.dataView.horizontalScrollBar().valueChanged.connect(
+            self.columnHeader.horizontalScrollBar().setValue
+        )
+        self.dataView.verticalScrollBar().valueChanged.connect(
+            self.indexHeader.verticalScrollBar().setValue
+        )
         # Scrolling in headers also scrolls the data table
-        self.columnHeader.horizontalScrollBar().valueChanged.connect(self.dataView.horizontalScrollBar().setValue)
-        self.indexHeader.verticalScrollBar().valueChanged.connect(self.dataView.verticalScrollBar().setValue)
+        self.columnHeader.horizontalScrollBar().valueChanged.connect(
+            self.dataView.horizontalScrollBar().setValue
+        )
+        self.indexHeader.verticalScrollBar().valueChanged.connect(
+            self.dataView.verticalScrollBar().setValue
+        )
 
         self.dataView.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.dataView.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
@@ -73,46 +65,53 @@ class DataFrameViewer(QtWidgets.QWidget):
         self.gridLayout.setContentsMargins(0, 0, 0, 0)
         self.gridLayout.setSpacing(0)
 
-        # Toggle level names
-        if not (any(df.columns.names) or df.columns.name):
-            self.columnHeader.verticalHeader().setFixedWidth(0)
-        if not (any(df.index.names) or df.index.name):
-            self.indexHeader.horizontalHeader().setFixedHeight(0)
-
-        # Add items to layout
+        # Add items to 4x4 grid layout
         self.gridLayout.addWidget(self.columnHeader, 0, 1, 1, 2)
         self.gridLayout.addWidget(self.indexHeader, 1, 0, 2, 2)
         self.gridLayout.addWidget(self.dataView, 2, 2, 1, 1)
         self.gridLayout.addWidget(self.dataView.horizontalScrollBar(), 3, 2, 1, 1)
         self.gridLayout.addWidget(self.dataView.verticalScrollBar(), 2, 3, 1, 1)
+        self.gridLayout.addWidget(self.columnHeaderNames, 0, 3, 1, 1)
+        self.gridLayout.addWidget(self.indexHeaderNames, 0, 0, 1, 1, Qt.AlignBottom)
 
         # These expand when the window is enlarged instead of having the grid squares spread out
         self.gridLayout.setColumnStretch(4, 1)
         self.gridLayout.setRowStretch(4, 1)
 
-        # These placeholders will ensure the size of the blank spaces beside our headers
-        self.gridLayout.addWidget(TrackingSpacer(ref_x=self.columnHeader.verticalHeader()), 3, 1, 1, 1)
-        self.gridLayout.addWidget(TrackingSpacer(ref_y=self.indexHeader.horizontalHeader()), 1, 2, 1, 1)
-        self.gridLayout.addItem(QtWidgets.QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Expanding), 0, 0, 1, 1)
+        self.gridLayout.addItem(
+            QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding), 0, 0, 1, 1, )
 
-        # Styling
-        for header in [self.indexHeader, self.columnHeader]:
-            header.setStyleSheet("background-color: white;"
-                                 "selection-color: black;"
-                                 "selection-background-color: #EAEAEA;")
+        self.columnHeaderNames.setDisabled(True)
 
-        self.dataView.setStyleSheet("background-color: white;"
-                                    "alternate-background-color: #F4F6F6;"
-                                    "selection-color: black;"
-                                    "selection-background-color: #BBDEFB;")
+        # Do this at top so sizeHints are calculated correctly
+        self.set_styles()
+        self.updateGeometry()
 
-        for item in [self.dataView, self.columnHeader, self.indexHeader]:
+    def set_styles(self):
+        for header in [self.indexHeader, self.columnHeader, self.indexHeaderNames, self.columnHeaderNames]:
+            header.setStyleSheet(
+                "background-color: white;"
+                "selection-color: black;"
+                "selection-background-color: #EAEAEA;"
+            )
+
+        self.dataView.setStyleSheet(
+            "background-color: white;"
+            "alternate-background-color: #F4F6F6;"
+            "selection-color: black;"
+            "selection-background-color: #BBDEFB;"
+        )
+        for item in [self.dataView, self.columnHeader, self.indexHeader, self.indexHeaderNames, self.columnHeaderNames]:
             item.setContentsMargins(0, 0, 0, 0)
             item.setStyleSheet(item.styleSheet() + "border: 0px solid black;")
             item.setItemDelegate(NoFocusDelegate())
 
-    def update_df(self, df):
-        self.df = df
+        for item in [self.indexHeaderNames, self.columnHeaderNames]:
+            item.setStyleSheet(item.styleSheet() + "color: grey;")
+
+    def __reduce__(self):
+        # This is so dataclasses.asdict doesn't complain about this being unpicklable
+        return "DataFrameViewer"
 
     def showEvent(self, event: QtGui.QShowEvent):
         """
@@ -145,7 +144,7 @@ class DataFrameViewer(QtWidgets.QWidget):
         """
         Set the size of column at column_index to fit its contents
         """
-        padding = 20
+        padding = 30
 
         self.columnHeader.resizeColumnToContents(column_index)
         width = self.columnHeader.columnWidth(column_index)
@@ -192,10 +191,7 @@ class DataFrameViewer(QtWidgets.QWidget):
             # This constrained width, with the flag of Qt.TextWordWrap
             # gets the height the cell would have to be to fit the text.
             constrained_rect = QtCore.QRect(0, 0, cell_width, 0)
-            h = self.dataView.fontMetrics().boundingRect(constrained_rect,
-                                                         Qt.TextWordWrap,
-                                                         text).height()
-
+            h = self.dataView.fontMetrics().boundingRect(constrained_rect, Qt.TextWordWrap, text).height()
             height = max(height, h)
 
         height += padding
@@ -211,35 +207,37 @@ class DataFrameViewer(QtWidgets.QWidget):
         QtWidgets.QWidget.keyPressEvent(self, event)
 
         if event.matches(QtGui.QKeySequence.Copy):
-            print('Ctrl + C')
             self.dataView.copy()
         if event.matches(QtGui.QKeySequence.Paste):
             self.dataView.paste()
-            print('Ctrl + V')
-        if event.key() == Qt.Key_P and (event.modifiers() & Qt.ControlModifier):
-            self.dataView.print()
-            print('Ctrl + P')
-        if event.key() == Qt.Key_D and (event.modifiers() & Qt.ControlModifier):
-            self.debug()
-            print('Ctrl + D')
-
-    def debug(self):
-        print(self.columnHeader.sizeHint())
-        print(self.dataView.sizeHint())
-        print(self.dataView.horizontalScrollBar().sizeHint())
+        # if event.key() == Qt.Key_P and (event.modifiers() & Qt.ControlModifier):
+        #     print("Ctrl + P")
+        # if event.key() == Qt.Key_D and (event.modifiers() & Qt.ControlModifier):
+        #     print("Ctrl + D")
 
     def data_changed(self):
         # Call dataChanged on all models for all data
-        for model in [self.dataView.model(), self.columnHeader.model(), self.indexHeader.model()]:
-            model.dataChanged.emit(model.index(0, 0), model.index(model.rowCount(), model.columnCount()))
+        for model in [
+            self.dataView.model(),
+            self.columnHeader.model(),
+            self.indexHeader.model(),
+            self.columnHeaderNames.model(),
+            self.indexHeaderNames.model(),
+        ]:
+            model.dataChanged.emit(QtCore.QModelIndex(), QtCore.QModelIndex())
 
 
 # Remove dotted border on cell focus.  https://stackoverflow.com/a/55252650/3620725
 class NoFocusDelegate(QtWidgets.QStyledItemDelegate):
-    def paint(self, QPainter, QStyleOptionViewItem, QModelIndex):
-        if QStyleOptionViewItem.state & QtWidgets.QStyle.State_HasFocus:
-            QStyleOptionViewItem.state = QStyleOptionViewItem.state ^ QtWidgets.QStyle.State_HasFocus
-        super().paint(QPainter, QStyleOptionViewItem, QModelIndex)
+    def paint(
+            self,
+            painter: QtGui.QPainter,
+            item: QtWidgets.QStyleOptionViewItem,
+            ix: QtCore.QModelIndex,
+    ):
+        if item.state & QtWidgets.QStyle.State_HasFocus:
+            item.state = item.state ^ QtWidgets.QStyle.State_HasFocus
+        super().paint(painter, item, ix)
 
 
 class DataTableModel(QtCore.QAbstractTableModel):
@@ -249,26 +247,28 @@ class DataTableModel(QtCore.QAbstractTableModel):
 
     def __init__(self, parent):
         super().__init__(parent)
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
 
     def headerData(self, section, orientation, role=None):
         # Headers for DataTableView are hidden. Header data is shown in HeaderView
         pass
 
     def columnCount(self, parent=None):
-        if type(self.parent().df) == pd.Series:
-            return 1
-        else:
-            return self.parent().df.columns.shape[0]
+        return self.pgdf.dataframe.columns.shape[0]
 
     def rowCount(self, parent=None):
-        return len(self.parent().df)
+        return len(self.pgdf.dataframe)
 
     # Returns the data from the DataFrame
     def data(self, index, role=QtCore.Qt.DisplayRole):
-        if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole or role == QtCore.Qt.ToolTipRole:
+        if (
+                role == QtCore.Qt.DisplayRole
+                or role == QtCore.Qt.EditRole
+                or role == QtCore.Qt.ToolTipRole
+        ):
             row = index.row()
             col = index.column()
-            cell = self.parent().df.iloc[row, col]
+            cell = self.pgdf.dataframe.iloc[row, col]
 
             # NaN case
             if pd.isnull(cell):
@@ -284,7 +284,7 @@ class DataTableModel(QtCore.QAbstractTableModel):
         elif role == QtCore.Qt.ToolTipRole:
             row = index.row()
             col = index.column()
-            cell = self.parent().df.iloc[row, col]
+            cell = self.pgdf.dataframe.iloc[row, col]
 
             # NaN case
             if pd.isnull(cell):
@@ -293,23 +293,20 @@ class DataTableModel(QtCore.QAbstractTableModel):
             return str(cell)
 
     def flags(self, index):
-        if store.settings.editable:
+        if self.parent().pgdf.settings.editable:
             return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
         else:
             return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
 
-    # Set data in the DataFrame. Required if table is editable
     def setData(self, index, value, role=None):
         if role == QtCore.Qt.EditRole:
             row = index.row()
             col = index.column()
             try:
-                self.parent().df.iat[row, col] = value
+                self.pgdf.edit_data(row, col, value)
             except Exception as e:
-                print(e)
+                logger.exception(e)
                 return False
-            self.dataChanged.emit(index, index)
-
             return True
 
 
@@ -320,6 +317,7 @@ class DataTableView(QtWidgets.QTableView):
 
     def __init__(self, parent):
         super().__init__(parent)
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
 
         # Create and set model
         model = DataTableModel(parent)
@@ -354,13 +352,19 @@ class DataTableView(QtWidgets.QTableView):
 
         if not columnHeader.hasFocus():
             selection = self.selectionModel().selection()
-            columnHeader.selectionModel().select(selection,
-                                                 QItemSelectionModel.Columns | QItemSelectionModel.ClearAndSelect)
+            columnHeader.selectionModel().select(
+                selection,
+                QtCore.QItemSelectionModel.Columns
+                | QtCore.QItemSelectionModel.ClearAndSelect,
+            )
 
         if not indexHeader.hasFocus():
             selection = self.selectionModel().selection()
-            indexHeader.selectionModel().select(selection,
-                                                QItemSelectionModel.Rows | QItemSelectionModel.ClearAndSelect)
+            indexHeader.selectionModel().select(
+                selection,
+                QtCore.QItemSelectionModel.Rows
+                | QtCore.QItemSelectionModel.ClearAndSelect,
+            )
 
     def copy(self):
         """
@@ -369,27 +373,42 @@ class DataTableView(QtWidgets.QTableView):
 
         # Get the bounds using the top left and bottom right selected cells
         indexes = self.selectionModel().selection().indexes()
-
         rows = [ix.row() for ix in indexes]
         cols = [ix.column() for ix in indexes]
 
-        df = self.parent().df.iloc[min(rows):max(rows) + 1, min(cols):max(cols) + 1]
+        df = self.pgdf.dataframe.iloc[min(rows): max(rows) + 1, min(cols): max(cols) + 1]
 
-        # If I try to use Pyperclip without starting new thread large values give access denied error
-        def thread_function(df):
-            df.to_clipboard(index=False, header=False)
-
-        threading.Thread(target=thread_function, args=(df,)).start()
+        # Special case for single-cell copy since df.to_clipboard appends extra newline
+        if df.shape == (1, 1):
+            clipboard = QtWidgets.QApplication.instance().clipboard()
+            value = str(df.iloc[0, 0])
+            clipboard.setText(value)
+        else:
+            # If I try to use Pyperclip without starting new thread large selections give access denied error
+            threading.Thread(target=lambda df: df.to_clipboard(index=False, header=False), args=(df,)).start()
 
     def paste(self):
-        # Set up clipboard object
-        app = QtWidgets.QApplication.instance()
-        if not app:
-            app = QtWidgets.QApplication(sys.argv)
-        clipboard = app.clipboard()
+        df_to_paste = pd.read_clipboard(sep=',|\t', header=None)
 
-        # TODO
-        print(clipboard.text())
+        # Get the bounds using the top left and bottom right selected cells
+        indexes = self.selectionModel().selection().indexes()
+        rows = [ix.row() for ix in indexes]
+        cols = [ix.column() for ix in indexes]
+
+        self.pgdf.paste_data(min(rows), min(cols), df_to_paste)
+
+        # Select the range of cells that were pasted
+        self.selectionModel().clearSelection()
+        temp = self.selectionMode()
+        self.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
+
+        for i in range(df_to_paste.shape[0]):
+            for j in range(df_to_paste.shape[1]):
+
+                self.selectionModel().select(self.model().index(min(rows) + i, min(cols) + j),
+                                             QtCore.QItemSelectionModel.Select)
+
+        self.setSelectionMode(temp)
 
     def sizeHint(self):
         # Set width and height based on number of columns in model
@@ -405,31 +424,29 @@ class DataTableView(QtWidgets.QTableView):
         for i in range(self.model().rowCount()):
             height += self.rowHeight(i)
 
-        return QSize(width, height)
+        return QtCore.QSize(width, height)
 
 
 class HeaderModel(QtCore.QAbstractTableModel):
-    """
-    Model for HeaderView
-    """
 
     def __init__(self, parent, orientation):
         super().__init__(parent)
         self.orientation = orientation
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
 
     def columnCount(self, parent=None):
         if self.orientation == Qt.Horizontal:
-            return self.parent().df.columns.shape[0]
+            return self.pgdf.dataframe.columns.shape[0]
         else:  # Vertical
-            return self.parent().df.index.nlevels
+            return self.pgdf.dataframe.index.nlevels
 
     def rowCount(self, parent=None):
         if self.orientation == Qt.Horizontal:
-            return self.parent().df.columns.nlevels
+            return self.pgdf.dataframe.columns.nlevels
         elif self.orientation == Qt.Vertical:
-            return self.parent().df.index.shape[0]
+            return self.pgdf.dataframe.index.shape[0]
 
-    def data(self, index, role=None):
+    def data(self, index, role=QtCore.Qt.DisplayRole):
         row = index.row()
         col = index.column()
 
@@ -437,32 +454,41 @@ class HeaderModel(QtCore.QAbstractTableModel):
 
             if self.orientation == Qt.Horizontal:
 
-                if type(self.parent().df.columns) == pd.MultiIndex:
-                    return str(self.parent().df.columns.values[col][row])
+                if type(self.pgdf.dataframe.columns) == pd.MultiIndex:
+                    return str(self.pgdf.dataframe.columns.values[col][row])
                 else:
-                    return str(self.parent().df.columns.values[col])
+                    return str(self.pgdf.dataframe.columns.values[col])
 
             elif self.orientation == Qt.Vertical:
 
-                if type(self.parent().df.index) == pd.MultiIndex:
-                    return str(self.parent().df.index.values[row][col])
+                if type(self.pgdf.dataframe.index) == pd.MultiIndex:
+                    return str(self.pgdf.dataframe.index.values[row][col])
                 else:
-                    return str(self.parent().df.index.values[row])
+                    return str(self.pgdf.dataframe.index.values[row])
+
+        if role == QtCore.Qt.DecorationRole:
+            if self.pgdf.sort_is_ascending:
+                icon = QtGui.QIcon(os.path.join(pandasgui.__path__[0], "images/sort-ascending.png"))
+            else:
+                icon = QtGui.QIcon(os.path.join(pandasgui.__path__[0], "images/sort-descending.png"))
+
+            if col == self.pgdf.column_sorted and row == self.rowCount() - 1 and self.orientation == Qt.Horizontal:
+                return icon
 
     # The headers of this table will show the level names of the MultiIndex
     def headerData(self, section, orientation, role=None):
         if role in [QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole]:
 
             if self.orientation == Qt.Horizontal and orientation == Qt.Vertical:
-                if type(self.parent().df.columns) == pd.MultiIndex:
-                    return str(self.parent().df.columns.names[section])
+                if type(self.pgdf.dataframe.columns) == pd.MultiIndex:
+                    return str(self.pgdf.dataframe.columns.names[section])
                 else:
-                    return str(self.parent().df.columns.name)
+                    return str(self.pgdf.dataframe.columns.name)
             elif self.orientation == Qt.Vertical and orientation == Qt.Horizontal:
-                if type(self.parent().df.index) == pd.MultiIndex:
-                    return str(self.parent().df.index.names[section])
+                if type(self.pgdf.dataframe.index) == pd.MultiIndex:
+                    return str(self.pgdf.dataframe.index.names[section])
                 else:
-                    return str(self.parent().df.index.name)
+                    return str(self.pgdf.dataframe.index.name)
             else:
                 return None  # These cells should be hidden anyways
 
@@ -474,6 +500,7 @@ class HeaderView(QtWidgets.QTableView):
 
     def __init__(self, parent: DataFrameViewer, orientation):
         super().__init__(parent)
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
 
         # Setup
         self.orientation = orientation
@@ -493,44 +520,37 @@ class HeaderView(QtWidgets.QTableView):
         self.viewport().installEventFilter(self)
 
         # Settings
-        self.setSizePolicy(QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum))
+        self.setSizePolicy(
+            QtWidgets.QSizePolicy(
+                QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum
+            )
+        )
         self.setWordWrap(False)
-        self.setFont(QtGui.QFont("Times", weight=QtGui.QFont.Bold))
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
         self.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
 
+        font = QtGui.QFont()
+        font.setBold(True)
+        self.setFont(font)
+
         # Link selection to DataTable
         self.selectionModel().selectionChanged.connect(self.on_selectionChanged)
         self.setSpans()
-        self.initSize()
+        self.init_size()
 
-        # Orientation specific settings
-        if orientation == Qt.Horizontal:
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)  # Scrollbar is replaced in DataFrameViewer
-            self.horizontalHeader().hide()
-            self.verticalHeader().setDisabled(True)
-            self.verticalHeader().setHighlightSections(False)  # Selection lags a lot without this
-
-        else:
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-            self.verticalHeader().hide()
-            self.horizontalHeader().setDisabled(True)
-
-            self.horizontalHeader().setHighlightSections(False)  # Selection lags a lot without this
+        self.horizontalHeader().hide()
+        self.verticalHeader().hide()
+        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
 
         # Set initial size
         self.resize(self.sizeHint())
 
-    def on_clicked(self, ix: QModelIndex):
+    def on_clicked(self, ix: QtCore.QModelIndex):
         # When a header is clicked, sort the DataFrame by that column
         if self.orientation == Qt.Horizontal:
-            df = self.parent().df
-
-            df_sorted = df.sort_values(df.columns[ix.column()], kind='mergesort')
-
-            self.parent().update_df(df_sorted)
-            self.parent().data_changed()
+            self.pgdf.sort_column(ix.column())
 
     # Header
     def on_selectionChanged(self):
@@ -543,31 +563,43 @@ class HeaderView(QtWidgets.QTableView):
             dataView = self.parent().dataView
 
             # Set selection mode so selecting one row or column at a time adds to selection each time
-            if self.orientation == Qt.Horizontal:  # This case is for the horizontal header
+            if (
+                    self.orientation == Qt.Horizontal
+            ):  # This case is for the horizontal header
                 # Get the header's selected columns
                 selection = self.selectionModel().selection()
 
                 # Removes the higher levels so that only the lowest level of the header affects the data table selection
-                last_row_ix = self.parent().df.columns.nlevels - 1
+                last_row_ix = self.pgdf.dataframe.columns.nlevels - 1
                 last_col_ix = self.model().columnCount() - 1
-                higher_levels = QtCore.QItemSelection(self.model().index(0, 0),
-                                                      self.model().index(last_row_ix - 1, last_col_ix))
+                higher_levels = QtCore.QItemSelection(
+                    self.model().index(0, 0),
+                    self.model().index(last_row_ix - 1, last_col_ix),
+                )
                 selection.merge(higher_levels, QtCore.QItemSelectionModel.Deselect)
 
                 # Select the cells in the data view
-                dataView.selectionModel().select(selection,
-                                                 QtCore.QItemSelectionModel.Columns | QtCore.QItemSelectionModel.ClearAndSelect)
+                dataView.selectionModel().select(
+                    selection,
+                    QtCore.QItemSelectionModel.Columns
+                    | QtCore.QItemSelectionModel.ClearAndSelect,
+                )
             if self.orientation == Qt.Vertical:
                 selection = self.selectionModel().selection()
 
                 last_row_ix = self.model().rowCount() - 1
-                last_col_ix = self.parent().df.index.nlevels - 1
-                higher_levels = QtCore.QItemSelection(self.model().index(0, 0),
-                                                      self.model().index(last_row_ix, last_col_ix - 1))
+                last_col_ix = self.pgdf.dataframe.index.nlevels - 1
+                higher_levels = QtCore.QItemSelection(
+                    self.model().index(0, 0),
+                    self.model().index(last_row_ix, last_col_ix - 1),
+                )
                 selection.merge(higher_levels, QtCore.QItemSelectionModel.Deselect)
 
-                dataView.selectionModel().select(selection,
-                                                 QtCore.QItemSelectionModel.Rows | QtCore.QItemSelectionModel.ClearAndSelect)
+                dataView.selectionModel().select(
+                    selection,
+                    QtCore.QItemSelectionModel.Rows
+                    | QtCore.QItemSelectionModel.ClearAndSelect,
+                )
 
         self.selectAbove()
 
@@ -575,10 +607,10 @@ class HeaderView(QtWidgets.QTableView):
     # This should happen after every selection change
     def selectAbove(self):
         if self.orientation == Qt.Horizontal:
-            if self.parent().df.columns.nlevels == 1:
+            if self.pgdf.dataframe.columns.nlevels == 1:
                 return
         else:
-            if self.parent().df.index.nlevels == 1:
+            if self.pgdf.dataframe.index.nlevels == 1:
                 return
 
         for ix in self.selectedIndexes():
@@ -594,8 +626,8 @@ class HeaderView(QtWidgets.QTableView):
                     self.setSelection(self.visualRect(ix2), QtCore.QItemSelectionModel.Select)
 
     # Fits columns to contents but with a minimum width and added padding
-    def initSize(self):
-        padding = 20
+    def init_size(self):
+        padding = 30
 
         if self.orientation == Qt.Horizontal:
             min_size = 100
@@ -612,7 +644,6 @@ class HeaderView(QtWidgets.QTableView):
                 self.setColumnWidth(col, new_width)
                 self.table.setColumnWidth(col, new_width)
         else:
-            max_size = 1000
             self.resizeColumnsToContents()
             for col in range(self.model().columnCount()):
                 width = self.columnWidth(col)
@@ -620,7 +651,7 @@ class HeaderView(QtWidgets.QTableView):
 
     # This sets spans to group together adjacent cells with the same values
     def setSpans(self):
-        df = self.parent().df
+        df = self.pgdf.dataframe
 
         # Find spans for horizontal HeaderView
         if self.orientation == Qt.Horizontal:
@@ -729,14 +760,16 @@ class HeaderView(QtWidgets.QTableView):
         if event.type() == QtCore.QEvent.MouseButtonPress:
             if self.orientation == Qt.Horizontal:
                 mouse_position = event.pos().x()
-            elif self.orientation == Qt.Vertical:
+            else:
                 mouse_position = event.pos().y()
 
             if self.over_header_edge(mouse_position) is not None:
                 self.header_being_resized = self.over_header_edge(mouse_position)
                 self.resize_start_position = mouse_position
                 if self.orientation == Qt.Horizontal:
-                    self.initial_header_size = self.columnWidth(self.header_being_resized)
+                    self.initial_header_size = self.columnWidth(
+                        self.header_being_resized
+                    )
                 elif self.orientation == Qt.Vertical:
                     self.initial_header_size = self.rowHeight(self.header_being_resized)
                 return True
@@ -751,16 +784,16 @@ class HeaderView(QtWidgets.QTableView):
         if event.type() == QtCore.QEvent.MouseButtonDblClick:
             if self.orientation == Qt.Horizontal:
                 mouse_position = event.pos().x()
-            elif self.orientation == Qt.Vertical:
+            else:
                 mouse_position = event.pos().y()
 
             # Find which column or row edge the mouse was over and auto size it
             if self.over_header_edge(mouse_position) is not None:
                 header_index = self.over_header_edge(mouse_position)
                 if self.orientation == Qt.Horizontal:
-                    self.parent.auto_size_column(header_index)
+                    self.parent().auto_size_column(header_index)
                 elif self.orientation == Qt.Vertical:
-                    self.parent.auto_size_row(header_index)
+                    self.parent().auto_size_row(header_index)
                 return True
 
         # Handle active drag resizing
@@ -773,17 +806,19 @@ class HeaderView(QtWidgets.QTableView):
             # If this is None, there is no drag resize happening
             if self.header_being_resized is not None:
 
-                size = self.initial_header_size + (mouse_position - self.resize_start_position)
+                size = self.initial_header_size + (
+                        mouse_position - self.resize_start_position
+                )
                 if size > 10:
                     if self.orientation == Qt.Horizontal:
                         self.setColumnWidth(self.header_being_resized, size)
-                        self.parent.dataView.setColumnWidth(self.header_being_resized, size)
+                        self.parent().dataView.setColumnWidth(self.header_being_resized, size)
                     if self.orientation == Qt.Vertical:
                         self.setRowHeight(self.header_being_resized, size)
-                        self.parent.dataView.setRowHeight(self.header_being_resized, size)
+                        self.parent().dataView.setRowHeight(self.header_being_resized, size)
 
                     self.updateGeometry()
-                    self.parent.dataView.updateGeometry()
+                    self.parent().dataView.updateGeometry()
                 return True
 
             # Set the cursor shape
@@ -817,14 +852,125 @@ class HeaderView(QtWidgets.QTableView):
             width = 2 * self.frameWidth()  # Account for border & padding
             for i in range(self.model().columnCount()):
                 width += self.columnWidth(i)
-        return QSize(width, height)
+        return QtCore.QSize(width, height)
 
     # This is needed because otherwise when the horizontal header is a single row it will add whitespace to be bigger
     def minimumSizeHint(self):
         if self.orientation == Qt.Horizontal:
-            return QSize(0, self.sizeHint().height())
+            return QtCore.QSize(0, self.sizeHint().height())
         else:
-            return QSize(self.sizeHint().width(), 0)
+            return QtCore.QSize(self.sizeHint().width(), 0)
+
+
+class HeaderNamesModel(QtCore.QAbstractTableModel):
+
+    def __init__(self, parent, orientation):
+        super().__init__(parent)
+        self.orientation = orientation
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
+
+    def columnCount(self, parent=None):
+        if self.orientation == Qt.Horizontal:
+            return 1
+        elif self.orientation == Qt.Vertical:
+            return self.pgdf.dataframe.index.nlevels
+
+    def rowCount(self, parent=None):
+        if self.orientation == Qt.Horizontal:
+            return self.pgdf.dataframe.columns.nlevels
+        elif self.orientation == Qt.Vertical:
+            return 1
+
+    def data(self, index, role=QtCore.Qt.DisplayRole):
+        row = index.row()
+        col = index.column()
+
+        if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.ToolTipRole:
+
+            if self.orientation == Qt.Horizontal:
+                val = self.pgdf.dataframe.columns.names[row]
+                if val is None:
+                    val = ""
+                return str(val)
+
+            elif self.orientation == Qt.Vertical:
+                val = self.pgdf.dataframe.index.names[col]
+                if val is None:
+                    val = "index"
+                return str(val)
+
+        if role == QtCore.Qt.DecorationRole:
+            if self.pgdf.sort_is_ascending:
+                icon = QtGui.QIcon(os.path.join(pandasgui.__path__[0], "images/sort-ascending.png"))
+            else:
+                icon = QtGui.QIcon(os.path.join(pandasgui.__path__[0], "images/sort-descending.png"))
+
+            if col == self.pgdf.index_sorted and self.orientation == Qt.Vertical:
+                return icon
+
+
+class HeaderNamesView(QtWidgets.QTableView):
+    def __init__(self, parent: DataFrameViewer, orientation):
+        super().__init__(parent)
+        self.pgdf: PandasGuiDataFrame = parent.pgdf
+
+        # Setup
+        self.orientation = orientation
+        self.setModel(HeaderNamesModel(parent, orientation))
+
+        self.clicked.connect(self.on_clicked)
+
+        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+
+        self.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
+        self.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
+
+        self.horizontalHeader().hide()
+        self.verticalHeader().hide()
+
+        self.setSelectionMode(self.NoSelection)
+
+        font = QtGui.QFont()
+        font.setBold(True)
+        self.setFont(font)
+        self.init_size()
+
+    def on_clicked(self, ix: QtCore.QModelIndex):
+        # When the index header name is clicked, sort the index by that level
+        if self.orientation == Qt.Vertical:
+            self.pgdf.sort_index(ix.column())
+
+    def init_size(self):
+        # Match vertical header name widths to vertical header
+        if self.orientation == Qt.Vertical:
+            for ix in range(self.model().columnCount()):
+                self.setColumnWidth(ix, self.columnWidth(ix))
+
+    def sizeHint(self):
+        if self.orientation == Qt.Horizontal:
+            width = self.columnWidth(0)
+            height = self.parent().columnHeader.sizeHint().height()
+        else:  # Vertical
+            width = self.parent().indexHeader.sizeHint().width()
+            height = self.rowHeight(0)
+
+        return QtCore.QSize(width, height)
+
+    def minimumSizeHint(self):
+        return self.sizeHint()
+
+    def rowHeight(self, row: int) -> int:
+        return self.parent().columnHeader.rowHeight(row)
+
+    def columnWidth(self, column: int) -> int:
+        if self.orientation == Qt.Horizontal:
+            if all(name is None for name in self.pgdf.dataframe.columns.names):
+                return 0
+            else:
+                return super().columnWidth(column)
+        else:
+            return self.parent().indexHeader.columnWidth(column)
 
 
 # This is a fixed size widget with a size that tracks some other widget
@@ -846,14 +992,15 @@ class TrackingSpacer(QtWidgets.QFrame):
 
 
 # Examples
-if __name__ == '__main__':
+if __name__ == "__main__":
     app = QtWidgets.QApplication(sys.argv)
 
-    from pandasgui.datasets import iris, flights, pokemon, multi_df, simple
+    from pandasgui.datasets import pokemon, mi_manufacturing
 
     # view = DataFrameViewer(pokemon)
     # view.show()
 
-    view2 = DataFrameViewer(pokemon)
+    view2 = DataFrameViewer(mi_manufacturing)
     view2.show()
-    app.exec_()
\ No newline at end of file
+
+    app.exec_()
